
# Load packages
library("dplyr"); library("survival"); library("KMsurv"); library("clinfun"); library("rootSolve")

# Import data
D<-read.table("**INSERT LOCAL PATH TO DATA.CSV**",fill=TRUE,header=TRUE,sep=",")

# Variables
# Period:  as defined in paper
# Sender:  subject id within-session
# Time:  time at which subject entered
# Counterpart:  within-session id of Sender's counterpart
# group:  id given to subject pair in-period
# firstmove:  first time of entry for pair (group)
# secondmove:  second time of entry for pair (group)
# Treat:  treatment
# session:  session id
#  Note, time, firstmove and secondmove values are scaled so that 60 is the latest time of entry
#  (i.e. the IC-10 and IC-280 treatments are rescaled in the dataset)

###############################
# Create Datasets

#1.  Create dataset of all subject-period pairs
T<- D%>%
	mutate(
		#Indicator variables for discrete vs. continuous vs. inertial (near) settings
		discrete=ifelse(Treat=="PD" | Treat=="L-PD",1,0), continuous=ifelse(Treat=="PC" | Treat=="L-PC",1,0), near=ifelse(discrete==0 | continuous==0,1,0),
		#Rescale first and second move time for each subject, period onto [0,100] interval
		firstmove=ifelse(discrete==1,firstmove*4*100/60,firstmove*100/60), secondmove=ifelse(discrete==1,secondmove*4*100/60,secondmove*100/60),
		#Rescale entry time onto [0,100] interval
		time=ifelse(discrete==1,time*4*100/60,time*100/60)
	) 


#Function for estimating median of product limit estimate
getmed<-function(x,y){
	sob<-Surv(x,y)
	temp <- read.table(textConnection(capture.output( survfit(sob~1)  )),skip=2,header=TRUE)
	temp$median
}

#2. Create dataset of subject-wise averages, including product limit estimates
S<-D%>%
	mutate(
		#Indicator taking value of 1 if subject's planned entry time was not censored by counterpart preemption
		observed=(time<=firstmove)*1,
		#Indicator for discrete-time treatments
		discrete=ifelse(Treat=="PD" | Treat=="L-PD",1,0)
	) %>%
	#Create separate measures for each unique subject
	group_by(Treat,session,Sender) %>%
	summarise(
		discrete=mean(discrete),
		#Get median of product limit estimate of first entry time
		pl=getmed(time,observed),
		#Calculate raw median of first entry time
		raw=median(time),
		#Calculate subject-wise latest observed first entry time across periods
		maxtime=max(time)
		)%>%
	mutate(
		#Rescale first and second move time measures for each subject onto [0,100] interval
		pl=ifelse(discrete==1,pl*4*100/60,pl*100/60),
		raw=ifelse(discrete==1,raw*4*100/60,raw*100/60),
		maxtime=ifelse(discrete==1,maxtime*4*100/60,maxtime*100/60)
	)

#Replace failed product limit estimates with raw median entry time as discussed in footnote 11
S[is.na(S$pl),]$pl<-S[is.na(S$pl),]$raw

#3. Create dataset of session-wise medians (for conducting fully independent non-parametric tests)
N<-S %>%
	group_by(Treat,session)%>%
	filter(!is.na(pl))%>%
	summarise(
		firstmove=median(pl)
	)


###############################
# Sections 4.1, 4.2:  Summary Statistics

# Median Product Limit Estimate of Entry Time, by Treatment
tapply(S[!is.na(S$pl),]$pl,S[!is.na(S$pl),]$Treat,FUN=median)


###############################
# Sections 4.1-4.3:  Hypothesis Tests

#1.  Treatment comparison of PC and PD reported in footnote 12
wilcox.test(N[N$Treat=="PC",]$firstmove,N[N$Treat=="PD",]$firstmove)

#2.  Test of monotonic relationship between inertia and entry time reported in footnote 13

# code treatment in ascending order of inertia
x<-N[N$Treat=="IC60" | N$Treat=="PC"|N$Treat=="IC10"|N$Treat=="IC280",]
x$group<-1; x[x$Treat=="IC60",]$group<-2; x[x$Treat=="IC280",]$group<-3; x[x$Treat=="PC",]$group<-4
#run two variations on Jonckheere test of monotonic effect of inertia on entry times
jonckheere.test(x$firstmove,x$group, alternative="two.sided")
jonckheere.test(x$firstmove,x$group, alternative="two.sided",nperm=8000)

#3.  Test of differences between main treatments and diagnostic treatments reported in footnote 18
wilcox.test(N[N$Treat=="PC",]$firstmove,N[N$Treat=="L-PC",]$firstmove)
wilcox.test(N[N$Treat=="PD",]$firstmove,N[N$Treat=="L-PD",]$firstmove)


###############################
# Generate Figure 3
	
layout(matrix(c(1,2),1,2,byrow=TRUE))

plot(density(T[T$Treat=="PD",]$firstmove),xlim=c(-2,50),col="darkgray",main="(a)",xlab="% of Period Elapsed",ylab="Density",lty=1,ylim=c(0,0.5))
lines(density(T[T$Treat=="PC",]$firstmove),col="black",xlim=c(0,40),main="Continuous Time",xlab="% of Period Elapsed",ylab="Density",lty=1)

legend("right",legend=c("PD","PC"),col=c("darkgray","black"),lty=c(1,1),cex=0.75,bg="white",box.lwd="white")

plot.ecdf(S[S$Treat=="PD",]$pl,do.points=FALSE,verticals=TRUE,col.hor="darkgray",col.vert="darkgray",xlim=c(-2,50),ylim=c(0,1),xlab="% of Period Elapsed",ylab="CDF",main="(b)",lty=1,col="black",col.01line = NULL)
lines(ecdf((S[S$Treat=="PC",]$pl)), col.01line=rgb(0,0,0,0),do.points=FALSE,verticals=TRUE,col.hor="black",col.vert="black",lty=1)


###############################
# Generate Figure 4

plot.ecdf(S[S$Treat=="PD",]$pl,do.points=FALSE,verticals=TRUE,col.hor="darkgray",col.vert="darkgray",xlim=c(-2,50),ylim=c(0,1),xlab="% of Period Elapsed",ylab="CDF",main="",lty=1,lwd=0.5)
lines(ecdf((S[S$Treat=="IC10",]$pl)), col.01line=rgb(0,0,0,0),do.points=FALSE,verticals=TRUE,col.hor="black",col.vert="black",lty=3)
lines(ecdf((S[S$Treat=="IC60",]$pl)), col.01line=rgb(0,0,0,0),do.points=FALSE,verticals=TRUE,col.hor="black",col.vert="black",lty=2,lwd=0.75)
lines(ecdf((S[S$Treat=="IC280",]$pl)), col.01line=rgb(0,0,0,0),do.points=FALSE,verticals=TRUE,col.hor="black",col.vert="black",lty=6)
lines(ecdf((S[S$Treat=="PC",]$pl)), col.01line=rgb(0,0,0,0),do.points=FALSE,verticals=TRUE,col.hor="black",col.vert="black",lty=1)
legend("right",legend=c("PD (Discrete)","IC-10 (High Inertia)","IC-60 (Med Inertia)","IC-280 (Low Inertia)","PC (Zero Inertia)"),col=c("darkgray","black","black","black","black"),cex=0.7,lty=c(1,3,2,6,1),bg="white",box.lwd="white")


###############################
# Section 4.3:  Calculate the earliest time at which the Basin of Attraction exceeds 0.5 (immediate entry is risk dominant) for Main Treatments

# Our game's payoff function for own entry time (ta), counterpart entry time (tb) and temptation parameter varied in diagnostic treatment (pif)
u<-function(ta,tb,pif){

	c<-1; pid<-2.4; pis<-2.16 #Payoff parameters held fixed in design

	ifelse(ta==tb,
			
			pid*((1-ta)/2)-(c*((1-ta)^2)),
			
			ifelse(ta<tb,
				((1-tb)/2)*(pid+pif*(tb-ta)*(1+(2/(1-tb))))-(c*((1-ta)^2)),
				((1-ta)/2)*( pid-pis*(ta-tb) )-(c*((1-ta)^2))
			)
	)

}

#Generic formula for calculating the basin of attraction
basin<-function(a,b,c,d){
	bo<-(a-c)/(a-c+d-b); bo<-ifelse(is.nan(bo),1,bo)
	ifelse(bo>=0 & bo<=1,bo,1)
}

#Calculating the basin of attraction for our game
BOA<-function(t,delta,pif){
	a<-u(t,t,pif); b<-u(t,min(t+delta,0.4),pif); c<-u(min(t+delta,0.4),t,pif); d<-u(0.4,0.4,pif)
	 basin(a,b,c,d)
}

#Calculating the earliest time of risk dominance for main treatments
sep.main<-sapply(
	c(1/15,0.5/10,0.5/60,0.5/280,0),
	FUN=function(z){
		if(sign(BOA(0,z,4)-0.5)==sign(BOA(0.4,z,4)-0.5)){
			ifelse(sign(BOA(0,z,4))>0,0,0.4)
		}else{
			uniroot(function(x)BOA(x,z,4)-0.5,c(0,0.4))$root
		}	
	})


###############################
# Section 4.4:  Calculate the earliest time at which the Basin of Attraction exceeds 0.5 (immediate entry is risk dominant) for Diagnostic Treatments, Friedman and Oprea (2012)


# Earliest time for perfectly discrete time games
boa.discrete<-function(pif){
	dates<-seq(0,6/15,1/15); boa<-sapply(dates,FUN=function(x)BOA(x,1/15,pif))
	min(dates[which(boa==min(boa[which(boa>=0.5)]))])
}

# Earliest time  for diagnostic treatments
sep.diagnostic<-c(
	boa.discrete(4), #PD
	boa.discrete(1.4), #L-PD
	0.4, #PC (basin of attraction is constant in perfection continuous time)
	0.4	# L-PC (basin of attraction is constant in perfectly continuous time)
	)


# Prisoner's Dilemma Payoff function
pdu<-function(ta,tb,tm,df){
	cp<-10;

	ifelse(ta==tb,
			
			ta*10 + (1-ta)*df,
			
			ifelse(ta<tb,
				ta*10 + (tb-ta)*tm + (1-tb-(tb-ta))*df,
				tb*10+(1-ta-(ta-tb))*df
			)
	)

}

#Calculating the basin of attraction for prisoner's dilemma
pdBOA<-function(t,delta,tm,df){
	a<-pdu(t,t,tm,df); b<-pdu(t,min(t+delta,1),tm,df); c<-pdu(min(t+delta,1),t,tm,df); d<-pdu(1,1,tm,df)
	 basin(a,b,c,d)
}


# Earliest time of risk dominance for the Continuous Prisoner's Dilemma Grid-N treatments
sep.pd<-sapply(
	c(2,4,8,15,30,60),
	FUN=function(z){
		if(sign(pdBOA(0,1/z,18,4)-0.5) == sign(pdBOA(1,1/z,18,4)-0.5)){
			ifelse(sign(pdBOA(0,1/z,18,4)-0.5)>0,0,1)
		}else{
			uniroot(function(x)pdBOA(x,1/z,18,4)-0.5,c(0,1))$root
		}	
	})

# Earliest time of risk dominance for Continuous Prisoner's Dilemma (Inertial) Continuous Treatments
60*uniroot(function(x)pdBOA(x,0.5/60,14,4)-0.5,c(0,1))$root #Easy
60*uniroot(function(x)pdBOA(x,0.5/60,18,4)-0.5,c(0,1))$root #Mix-A
60*uniroot(function(x)pdBOA(x,0.5/60,14,8)-0.5,c(0,1))$root #Mix-B
60*uniroot(function(x)pdBOA(x,0.5/60,18,8)-0.5,c(0,1))$root #Hard



###############################
# Generate Figure 5

layout(matrix(c(1,2,1,3), 2, 2, byrow = TRUE))

# Panel (a): Main Data
observed<-0.01*c(median(S[S$Treat=="PD",]$pl),median(S[S$Treat=="IC10",]$pl),median(S[S$Treat=="IC60",]$pl),median(S[S$Treat=="IC280",]$pl),median(S[S$Treat=="PC",]$pl))

plot(1:5,observed,type="o",ylim=c(0,0.4),xaxt="n",xlab="Treatment",ylab="Time",lty=1,cex=1,pch=20,main="(a)  Main Treatments")
lines(1:5,sep.main,type="o",col="darkgray",cex=1,pch=1,lty=2)
axis(side=1,at=1:5,labels=c("PD","IC-10","IC-60","IC-280","PC"))
legend("topleft", legend=c("First Time of Risk Dominance","Median Entry Time"),col=c("gray","black"),lty=c(2,1),cex=c(1,1),pch=c(1,20),
box.lwd="white")

# Panel (b):  Diagnostics Treatments
observed<-0.01*c(median(S[S$Treat=="PD",]$pl),median(S[S$Treat=="L-PD",]$pl),median(S[S$Treat=="PC",]$pl),median(S[S$Treat=="L-PC",]$pl))
plot(1:4,observed,type="o",ylim=c(0,0.4),xaxt="n",xlab="Treatment",ylab="Time",lty=1,cex=1,pch=20,main="(b)  Diagnostic Treatments")
lines(1:4,sep.diagnostic,type="o",col="darkgray",cex=1,pch=1,lty=2)
axis(side=1,at=1:4,labels=c("PD","L-PD","PC","L-PC"))
legend("topleft", legend=c("First Time of Risk Dominance","Median Entry Time"),col=c("darkgray","black"),lty=c(2,1),cex=c(1,1),pch=c(1,20),
box.lwd="white")

# Panel (c):  Friedman/Oprea Data
# import Friedman/Oprea Data
A <- read.table("**INSERT LOCAL PATH TO FriedmanOprea.csv**",,header=TRUE,sep=",")
# calculate latest time of cooperation for Friedman/Oprea Data
D<-A%>%filter(inst=="L",x==18,y==4)%>%
	group_by(group)%>%mutate(coop=(P1S==1 & P2S==1), Tick=Tick+1, rate=mean(coop), cooptime=max(coop*Tick), thresh=cooptime/subp)%>%
	filter(Tick==1)%>%ungroup()
#  calculate medians for each lag level
cutoffs<-D%>%group_by(subp)%>%
	summarise(thresh=median(thresh),rate=median(rate))
#  plot data as panel (c)
plot(cutoffs$subp,cutoffs$thresh-0.003,type="o",ylim=c(0,1),xlab="Gridpoints",ylab="Time",main="(c)  Continuous Prisoner's Dilemma",lty=1,cex=1,pch=20)
lines(cutoffs$subp,sep.pd,type="o",col="darkgray",cex=1,pch=1,lty=2)
legend("right", legend=c("First Time of Risk Dominance","Estimated Threshold Time"),col=c("gray","black"),lty=c(2,1),cex=c(1,1),pch=c(1,20),
box.lwd="white")


###############################
# Section 4.0:  Second Mover Response time, inertia

SM<- D[D$Period>3,] %>%
	mutate(
		protocol=ifelse(Treat=="PD" | Treat=="L-PD","PD",ifelse(Treat=="PC" | Treat=="L-PC","PC","IC")),
		response=secondmove-firstmove
	)

#Quantile of lags for each timing protocol
tapply(SM$response,SM$protocol,FUN=function(x)quantile(x,seq(0,1,0.01)))


###############################
# Footnote 11:  Robustness on PL Measures

#Alternate treatment of failed product limit estimates, replaced with latest observed entry time.
S$upperpl<-S$pl; S[is.na(S$upperpl),]$upperpl<-S[is.na(S$upperpl),]$maxtime
#Median Upper Product Limit Estimates
tapply(S$upperpl,S$Treat,FUN=median)




